@@ -27,6 +27,7 @@ import os
 import subprocess
 import urlparse
 from cStringIO import StringIO
+import sys
 
 import boto.s3.key
 
@@ -104,7 +105,6 @@ def smart_open(uri, mode="rb"):
             return S3OpenWrite(outbucket, outkey)
         else:
             raise NotImplementedError("write mode not supported for %r scheme", parsed_uri.scheme)
-
     else:
         raise NotImplementedError("unknown file mode %s" % mode)
 
@@ -188,7 +188,6 @@ class S3OpenRead(object):
         if self.read_key is None:
             raise KeyError(parsed_uri.key_id)
 
-
     def __iter__(self):
         s3_connection = boto.connect_s3(
             aws_access_key_id=self.parsed_uri.access_id,
@@ -209,8 +208,8 @@ class S3OpenRead(object):
 
         """
         if size < 0 or not size:
-            # for compatibility with standard Python `read(negative)` = read the rest of the file
-            # XXX boto would read *from the start* if given size=-1
+            # For compatibility with standard Python, `read(negative)` = read the rest of the file.
+            # Otherwise, boto would read *from the start* if given size=-1.
             size = 0
         return self.read_key.read(size)
 
@@ -284,7 +283,11 @@ def file_smart_open(fname, mode='rb'):
     _, ext = os.path.splitext(fname)
 
     if ext == '.bz2':
-        from bz2 import BZ2File
+        PY2 = sys.version_info[0] == 2
+        if PY2:
+            from bz2file import BZ2File
+        else:
+            from bz2 import BZ2File
         return make_closing(BZ2File)(fname, mode)
 
     if ext == '.gz':
@@ -352,28 +355,33 @@ class S3OpenWrite(object):
     def seek(self, offset, whence=None):
         raise NotImplementedError("seek() not implemented yet")
 
+    def close(self):
+        buff = "".join(self.lines)
+        if buff:
+            logger.info("uploading last part #%i, %i bytes (total %.3fGB)" % (self.parts, len(buff), self.total_size / 1024.0 ** 3))
+            self.mp.upload_part_from_file(StringIO(buff), part_num=self.parts + 1)
+            logger.debug("upload of last part #%i finished" % self.parts)
+
+        if self.total_size:
+            self.mp.complete_upload()
+        else:
+            # AWS complains with "The XML you provided was not well-formed or did not validate against our published schema"
+            # when the input is completely empty => abort the upload, no file created
+            # TODO: or create the empty file some other way?
+            logger.info("empty input, ignoring multipart upload")
+            self.outbucket.cancel_multipart_upload(self.mp.key_name, self.mp.id)
+
     def __enter__(self):
         return self
 
     def __exit__(self, type, value, traceback):
         try:
-            buff = "".join(self.lines)
-            if buff:
-                logger.info("uploading last part #%i, %i bytes (total %.3fGB)" % (self.parts, len(buff), self.total_size / 1024.0 ** 3))
-                self.mp.upload_part_from_file(StringIO(buff), part_num=self.parts + 1)
-                logger.debug("upload of last part #%i finished" % self.parts)
-
-            if self.total_size:
-                self.mp.complete_upload()
-            else:
-                # AWS complains with "The XML you provided was not well-formed or did not validate against our published schema"
-                # when the input is completely empty => abort the upload, no file created
-                # TODO: or create the empty file some other way?
-                logger.info("empty input, ignoring multipart upload")
-                self.outbucket.cancel_multipart_upload(self.mp.key_name, self.mp.id)
-        except:  # including KeyboardInterrupt
+            self.close()
+        except:
             logger.exception("encountered error while terminating multipart upload; attempting cancel")
             self.outbucket.cancel_multipart_upload(self.mp.key_name, self.mp.id)
+            logger.info("cancel completed")
+            raise
 
 
 def s3_iter_bucket_process_key(key):
@@ -382,7 +390,6 @@ def s3_iter_bucket_process_key(key):
     of pickling visibility.
 
     """
-    logger.info("s3_iter_bucket_process_key")
     return key, key.get_contents_as_string()
 
 
